//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef INFORMATION_H
#define INFORMATION_H

#include <Config.hpp>

#include <QEvent>
#include <QString>
#include <QString>
#include <QTextStream>

class QMainWindow;

#define SETSTATUS(msg)                             \
  do {                                             \
    QString __status_msg;                          \
    {                                              \
      QTextStream __status_msgstrm(&__status_msg); \
      __status_msgstrm << msg;                     \
    }                                              \
    mgx::Information::print(__status_msg);         \
  } while(false)

#define DEBUG_OUTPUT(msg) \
  if(DEBUG)               \
    do {                  \
      Information::out << msg;   \
    } while(false)

namespace mgx {
  mgx_EXPORT extern bool DEBUG;

  namespace Information {
    mgx_EXPORT extern QTextStream out;
    mgx_EXPORT extern QTextStream err;
    
    struct mgx_EXPORT Event : public QEvent {
      Event(const QString& msg) : QEvent(QEvent::User), message(msg) {}
    
      QString message;
    };
    
    mgx_EXPORT void setMainWindow(QMainWindow* wnd);
    mgx_EXPORT void setStatus(const QString& text);
    mgx_EXPORT void print(const QString& text);
    mgx_EXPORT void print(const std::string text);
  }
}
#endif
