//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef MISC_H
#define MISC_H
/**
 * \file Misc.hpp
 *
 * Misc. definitions and utilities
 */
#include <Config.hpp>
#include <GL.hpp>

#include <Color.hpp>
#include <EdgeData.hpp>
#include <MGXViewer/qglviewer.h>
#include <Vector.hpp>
#include <VertexData.hpp>
#include <VVGraph.hpp>

#include <QDir>
#include <QList>
#include <QtGui>

namespace mgx {

static const QString UM(QString::fromWCharArray(L"\xb5m"));             // um
static const QString UM2(QString::fromWCharArray(L"\xb5m\xb2"));        // um^2
static const QString UM3(QString::fromWCharArray(L"\xb5m\xb3"));        // um^3
static const QString UM_1(QString::fromWCharArray(L"\xb5m\x207B\xb9")); // um^-1
static const QString UM_2(QString::fromWCharArray(L"\xb5m\x207B\xb2")); // um^-2

#ifdef _MSC_VER
template class mgx_EXPORT graph::Vertex<VertexData>;
template class mgx_EXPORT graph::Edge<EdgeData>;
template class mgx_EXPORT graph::VVGraph<VertexData, EdgeData>;
#endif

// Type of the VV graph
typedef graph::VVGraph<VertexData, EdgeData> vvgraph;

// Type of a vertex
typedef vvgraph::vertex_t vertex;

// Type of an edge
typedef vvgraph::edge_t edge;

using qglviewer::Quaternion;
typedef util::Vector<2, int> Point2i;
typedef util::Vector<3, GLuint> Point3GLui;
typedef util::Vector<4, GLuint> Point4GLui;
typedef util::Vector<3, GLubyte> Point3GLub;
typedef util::Vector<4, GLubyte> Point4GLub;
typedef util::Color<float> Colorf;
typedef util::Vector<3, float> Point3f;

namespace util {
/// Returns resource directory
mgx_EXPORT QDir resourcesDir();
/// Returns user processes directory
mgx_EXPORT QDir userProcessesDir();
/// Returns processes directory
mgx_EXPORT QList<QDir> processesDirs();
/// Returns the libraries directory
mgx_EXPORT QDir libsDir();
/// Returns the includes directory
mgx_EXPORT QDir includesDir();
/// Returns the documentation directory
mgx_EXPORT QDir docsDir();

/// Map unique colors to indices
inline Point3GLub vMapColor(uint u)
{
  u++;
  return (Point3GLub(u / (256 * 256), u / 256 % 256, u % 256));
}

/// Map unique colors to indices
inline uint vMapColor(Point3GLub& p) {
  return ((int(p.x()) * 256 * 256 + int(p.y()) * 256 + int(p.z())) - 1);
}

/// Create representation of a string that can be written in a single line, without spaces
inline QString shield(QString s)
{
  s.replace("\\", "\\\\");
  s.replace(" ", "\\s");
  s.replace("\n", "\\n");
  s.replace("\t", "\\t");
  s.replace("\"", "\\\"");
  s.replace("\'", "\\\'");
  return s;
}

/// Retrieve a string that has been shielded with shield
inline QString unshield(QString s)
{
  s.replace("\\\'", "\'");
  s.replace("\\\"", "\"");
  s.replace("\\s", " ");
  s.replace("\\n", "\n");
  s.replace("\\t", "\t");
  s.replace("\\\\", "\\");
  return s;
}

/// Shield a string to send it to the python interpreter
inline QString shield_python(QString s)
{
  s.replace("\\", "\\\\");
  s.replace("\n", "\\n");
  s.replace("\t", "\\t");
  s.replace("\"", "\\\"");
  s.replace("\'", "\\\'");
  return s;
}

/// Retrieve a string that is retrieved from the python representation
inline QString unshield_python(QString s)
{
  s.replace("\\\'", "\'");
  s.replace("\\\"", "\"");
  s.replace("\\n", "\n");
  s.replace("\\t", "\t");
  s.replace("\\\\", "\\");
  return s;
}

} // namespace util
} // namespace mgx
#endif
