//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
//
#ifndef AUTO_TRIM_HPP
#define AUTO_TRIM_HPP

#include <Process.hpp>

namespace mgx {
namespace process {
/**
 * \class AutoTrim AutoTrim.hpp <AutoTrim.hpp>
 *
 * Trim stack boundary box (BBox) to keep only non-empty part. A voxel is
 * considered empty if its intensity is less or equal to the threshold.
 *
 * \ingroup StackProcess
 */
class AutoTrim : public StackProcess {
public:
  AutoTrim(const StackProcess& process)
    : Process(process)
    , StackProcess(process)
  {
  }

  bool operator()(const QStringList& parms)
  {
    if(!checkState().stack(STACK_ANY))
      return false;

    return (*this)(currentStack(), currentStack()->currentStore(), parms[0].toInt());
  }

  bool operator()(Stack* stack, Store* store, int threshold);

  QString folder() const {
    return "Canvas";
  }
  QString name() const {
    return "AutoTrim";
  }
  QString description() const
  {
    return "Trim stack boundary box (BBox) to keep only non-empty part.\n"
           "A voxel is considered empty if its intensity is less or equal to the threshold.";
  }
  QStringList parmNames() const {
    return QStringList() << "Threshold";
  }
  QStringList parmDescs() const {
    return QStringList() << "Voxels below this threshold are considered empty.";
  }
  QStringList parmDefaults() const {
    return QStringList() << "0";
  }
  QIcon icon() const {
    return QIcon(":/images/AutoTrim.png");
  }
};
} // namespace process
} // namespace mgx
#endif
