//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef ITKREGISTRATION_HPP
#define ITKREGISTRATION_HPP

#include <ITKConfig.hpp>
#include <ITKProcess.hpp>

namespace mgx {
namespace process {

class mgxITK_EXPORT ITKDemonsRegistration : public StackProcess {
public:
  ITKDemonsRegistration(const StackProcess& process) : Process(process), StackProcess(process) {}

  bool operator()(const QStringList& parms)
  {
    Stack* stk = currentStack();
    Stack* ref_stk = stack(stk->id() == 0 ? 1 : 0);
    Store* ref = ref_stk->currentStore();
    return operator()(ref, stk, parms[0], stringToBool(parms[1]), parms[2].toInt(), parms[3].toInt(),
                      parms[4].toInt(), parms[5].toFloat());
  }

  bool operator()(const Store* reference, Stack* image, QString output, bool thresholdAtMean, int nbHistLevels,
                  int nbMatchPoints, int nbIterations, float sigma);

  QString name() const {
    return "ITK Demons Registration";
  }
  QString folder() const {
    return "ITK/Registration";
  }
  QString description() const {
    return "Register the current image with the current one on the other stack";
  }

  QStringList parmNames() const
  {
    return QStringList() << "Deformation File"
                         << "Threshold at mean intensity"
                         << "# Histogram levels"
                         << "# Match points"
                         << "# Iterations"
                         << "Sigma";
  }
  QStringList parmDescs() const
  {
    return QStringList() << "Deformation File"
                         << "Threshold at mean intensity"
                         << "# Histogram levels"
                         << "# Match points"
                         << "# Iterations"
                         << "Sigma";
  }
  QStringList parmDefaults() const
  {
    return QStringList() << "deformation.vtk"
                         << "Yes"
                         << "1024"
                         << "7"
                         << "150"
                         << "1.0";
  }

  ParmChoiceMap stringChoice() const
  {
    ParmChoiceMap map;
    map[0] = booleanChoice();
    return map;
  }
  QIcon icon() const {
    return QIcon(":/images/Blur.png");
  }
};
}
}

#endif // ITKREGISTRATION_HPP
