//
// This file is part of MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// If you use MorphoGraphX in your work, please cite:
//   http://dx.doi.org/10.7554/eLife.05864
//
// MorphoGraphX is free software, and is licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef ITKSOURCE_HPP
#define ITKSOURCE_HPP

#include <QObject>
#include <ITKConfig.hpp>
#include <ITKProcess.hpp>
#include <ui_ImportStack.h>
#include <itkImageFileReader.h>

namespace mgx {
namespace process {
typedef itk::Image<unsigned short int, 5> U5ImageType;
typedef itk::ImageFileReader<U5ImageType> ReaderType;

class mgxITK_EXPORT ITKImageReader : public QObject, public StackProcess {
  Q_OBJECT
public:
  ITKImageReader(const StackProcess& process) 
    : Process(process), QObject(), StackProcess(process) {}

  bool initialize(QStringList& parms, QWidget* parent);

  bool operator()(const QStringList& parms);
  bool operator()(Stack* stack, Store* store, QString filename);

  QString name() const {
    return "ITK Image Reader";
  }
  QString folder() const {
    return "ITK/System";
  }
  QString description() const {
    return "Read a 3D image from a single file";
  }
  QStringList parmNames() const
  {
    return QStringList() << "Filename"
                         << "Store"
                         << "Stack number";
  }
  QStringList parmDescs() const
  {
    return QStringList() << "Filename"
                         << "Store"
                         << "Stack number";
  }
  QStringList parmDefaults() const
  {
    return QStringList() << ""
                         << "Main"
                         << "0";
  }
  ParmChoiceMap stringChoice() const
  {
    ParmChoiceMap map;
    map[1] = storeChoice();
    return map;
  }
  QIcon icon() const {
    return QIcon(":/images/open.png");
  }

protected:
  void LoadSeries(bool next);

protected slots:
  void NextSeriesSlot() {
    LoadSeries(true);
  };
  void PrevSeriesSlot() {
    LoadSeries(false);
  };

protected:
  QString filename;
  Point5i size;
  Point5f step;

  QString pixelType;
  int componentSize;
  int components;

  int selectSeries;
  int selectComponent;
  int selectChannel;
  int selectTimePoint;

  Ui_ImportStackDialog* ui;
  itk::ImageIOBase::Pointer imageIO;
};
}
}

#endif
