//
// This file is part of 3DCellAtlas. 
// Copyright (C) 2015 George W. Bassel and collaborators.
//
// If you use 3DCellAtlas in your work, please cite:
//   http://dx.doi.org/10.1105/tpc.15.00175
//
// 3DCellAtlas is an AddOn for MorphoGraphX - http://www.MorphoGraphX.org
// Copyright (C) 2012-2015 Richard S. Smith and collaborators.
//
// 3DCellAtlas and MorphoGraphX are free software, and are licensed under under the terms of the 
// GNU General (GPL) Public License version 2.0, http://www.gnu.org/licenses.
// 
#ifndef RootCellAnalyzing_H
#define RootCellAnalyzing_H

#include <CellAtlasConfig.hpp>

#include "RootCellProcessing.hpp"
#include <math.h>
#include "Progress.hpp"

#define PI 3.14159265

using namespace std;

namespace mgx {
namespace process {

class  CellAtlas_EXPORT RootCellAnalyzing 
{
public:
  // estimates the size of the cell in a given direction
  double estimateCellLength(int currentLabel, Point3f coordCellCentroid, Point3f& dirVec, 
                                                                   triVector& cellTriangles);

  // analyzes the root based on: Bezier, segmeneted mesh and surface mesh
  // creates multiple outputs:
  // cellTriangles: map of label -> unique triangles
  // cellCentroids: center of all cells
  bool analyzeEmbryo(std::vector<int>& uniqueLabels, const vvgraph& segmentedMesh, 
    const vvgraph& surfaceMesh, bezierMap& bMap, bezierMap& diffbMap, labelVertexMap& lvMap, 
    RootCellProcessing& rootDataBox, int firstLabel, double minVolume);

private:
  labelBoolMap badCells;
};
}
}

#endif
